/* Copyright (c) 1996 University of Utah.  All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for NON-COMMERCIAL purposes and without fee is
 * hereby granted provided that this copyright notice appears in all
 * copies.
 *
 * See http://www.cs.utah.edu/~hamlet for more educational resources.
 *
 * The University of Utah MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT
 * THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 * NON-INFRINGEMENT. The University of Utah SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/*
<applet 
    CODEBASE=".."
    CODE="hamlet.AppButton.class" NAME="SlidingBlock.SlidingBlock"
    width=160 height=25 align=absbottom>
    <param name=LABEL value="Sliding Block Simulation">
    A Java applet should appear here
</applet>
*/


/************************************************************************
 *
 * SlidingBlock class
 *
 *      FileName:          SlidingBlock.java
 *      Written by:        Joseph Turner
 *      Modified by:       Joe Zachary
 *                         Elizabeth Odekirk
 *
 ************************************************************************/ 


package SlidingBlock;

import java.awt.*;
import java.io.*;
import hamlet.*;



public class SlidingBlock extends AppFrame implements Runnable {

    Slider topSlider,        // The two sliders.  topSlider always exists;
	   botSlider;        // botSlider only sometimes

    ScrollableNumbers timeField;   // Simulated time

    CheckboxMenu speedMenu;  // Sets speedratio
    CheckboxMenu scaleMenu;  // Sets distance scale

    MenuItem go;             // Start animation
    MenuItem stop;           // Stop animation
    MenuItem one;            // Show one slider
    MenuItem two;            // Show two sliders
  
    Thread animatorThread = null;   // Thread that controls animation

    double speedratio = 1;   // Ratio of animated time to real time

    Panel topPanel,          // Top slider with friction control
	  botPanel,          // Bottom slider with friction control
	  sliderPanel;       // Contains top and bottom sliders


    //--------------------------------------------------
    // Constructor
    //--------------------------------------------------
  
    public SlidingBlock () {

	// Title

	setTitle("Sliding Block 1.0");


	// Menu bar

	MenuBar menuBar = getMenuBar();

	// Animation/display controls

	Menu animMenu = new Menu("Slide");
	go = new MenuItem("Start");
	stop = new MenuItem("Stop");
	two = new MenuItem("Two Ramps");
	one = new MenuItem("One Ramp");
	go.enable(true);
	stop.enable(false);
	animMenu.add(go);
	animMenu.add(stop);
	animMenu.add(two);
	animMenu.add(one);
	one.enable(false);
	
	// Time/space scaling controls

	Menu scalingMenu = new Menu("Scale");

	scaleMenu = new CheckboxMenu("Distance");
	scaleMenu.add("1 cm/pixel");
	scaleMenu.add("10 cm/pixel");
	scaleMenu.add("1 m/pixel");
	scaleMenu.add("10 m/pixel");

	speedMenu = new CheckboxMenu("Time");
	speedMenu.add("8   x speed");
	speedMenu.add("4   x speed");
	speedMenu.add("2   x speed");
	speedMenu.add("1   x speed");
	speedMenu.add("1/2 x speed");

	scalingMenu.add(scaleMenu);
	scalingMenu.add(speedMenu);

	menuBar.add(animMenu);
	menuBar.add(scalingMenu);


	// Layout

        setLayout(new BorderLayout());


	// Time goes at the top

	timeField = new ScrollableNumbers(0.0,
					  0.1,
					  1.0,
					  0,
					  1000,
					  6,
					  ScrollableNumbers.LEFT,
					  "Time ",
					  this);
	add("North", timeField);


	// Next come the slider panels

	sliderPanel = new Panel();
	sliderPanel.setLayout(new GridLayout(1,1));

	topSlider = new Slider();
	topPanel = makeSliderPanel(topSlider, 0);

	botPanel = null;
	botSlider = null;

	sliderPanel.add(topPanel);
	add("Center", sliderPanel);


	// Set the menu defaults
	
	speedMenu.setChoice(3);
	scaleMenu.setChoice(2);

	pack();

	helpText =
	    " This applet illustrates the motion of a block sliding\n" +
	    " on an inclined plane in the presence of friction as\n" +
	    " discussed in Chapter 12 of \"Introduction to Scientific\n" +
	    " Programming\" by Joseph L. Zachary.\n\n" +

	    " File Menu\n" +
	    "  + Reset:  Resets block to top of ramp\n" +
	    "  + Hide:  Removes the applet from the display\n" +
	    "  + Quit:  Terminates the applet\n\n" +

	    " Sliding Menu\n" +
	    "  + Start:  Starts sliding block animation\n" +
	    "  + Stop:  Stops sliding block animation\n" +
	    "  + Two Ramps:  Displays two ramps\n" +
	    "  + One Ramp:  Displays one ramp\n\n" +

	    " Scaling Menu\n" +
	    "  + Distance:  Relates actual ramp length in pixels to\n" +
	    "               simulated length in meters\n" +
	    "  + Time:  Relates real time to simulated time\n\n" +

	    " Help Menu\n" +
	    "  + With Applet:  Displays this help dialog\n\n" +

	    " Time\n" +
	    "  + Displays elapsed time in simulation.  The time\n" +
	    "    can also be modified directly to step the block.\n\n" +

	    " Friction\n" +
	    "  + Modifies the coefficient of friction of the block\n\n" +

	    " Mouse Actions\n" +
	    "  + Drag the lower end of the ramp to change its length\n" +
	    "    and angle\n\n" +

	    " Displayed Information\n" +
	    "  + The simulated length of the ramp in meters and its\n" +
	    "    angle in degrees are displayed in the upper right\n" +
	    "    corner\n" +
	    "  + The distance in meters of the back edge of the block\n" +
	    "    from the top of the ramp is displayed at the bottom\n" +
	    "  + The color of the block varies from blue to red as the\n" +
	    "    coefficient of friction increases\n" +
	    "  + Horizontal and vertical indicators highlight the\n" +
	    "    position of the block\n\n" +

            " (24Sep96)";

    }



    //--------------------------------------------------
    // Creates a slider panel given a slider and the initial
    // value for the coefficient of friction.  The panel
    // contains a slider, an updatable distance label, and
    // a friction input area.
    //--------------------------------------------------

    private Panel makeSliderPanel (Slider slider, double fric) {

	// Friction input

	ScrollableNumbers friction =
	    new ScrollableNumbers(fric,
				  0.1,
				  0,
				  0.0,
				  5,
				  4,
				  ScrollableNumbers.LEFT,
				  "Friction ",
				  slider);

	// Distance label

	NumericLabel distance =
	    new NumericLabel("Distance", 0, Label.LEFT, "m");

	distance.preferredWidth(7);

	// Tell the slider about them
	
	slider.setControls(friction, distance);

	// Create the panel and lay everything out

	Panel panel = new Panel();
	panel.setLayout(new BorderLayout(5,2));
	panel.add("Center", slider);
		Panel controlPanel = new Panel();

	Panel controls = new Panel();
	controls.setLayout(new FlowLayout(FlowLayout.CENTER));
	controls.add(distance);
	controls.add(friction);
	panel.add("South", controls);

	return(panel);

    }


	
    //--------------------------------------------------
    // Stops any ongoing animation and resets both sliders.  It's
    // also a good time to do some garbage collection.
    //--------------------------------------------------

    public synchronized void reset() {
	stop();
	topSlider.reset();
	if (botSlider != null) botSlider.reset();
	timeField.setDoubleText(0);
	System.gc();
    }
   

    
    //--------------------------------------------------
    // Deals with menu actaions
    //--------------------------------------------------
   
    public synchronized boolean action (Event theEvent, Object arg) {

      if (super.action(theEvent, arg)) return (true);

      else if (theEvent.target instanceof CheckboxMenuItem) {

	  // Time scaling

	  if (speedMenu.update(theEvent.target)) {
	      String s = (String) arg;
	      if (s.equals("1   x speed"))
		  speedratio = 1;
	      else if (s.equals("2   x speed"))
		  speedratio = 2;
	      else if (s.equals("4   x speed"))
		  speedratio = 4;
	      else if (s.equals("8   x speed"))
		  speedratio = 8;
	      else if (s.equals("1/2 x speed"))
		  speedratio = 0.5;
	      return(true);
	  }

	  // Distance scaling

	  else if (scaleMenu.update(theEvent.target)) {
	      double scale = 1;
	      String s = (String) arg;
	      if (s.equals("1 cm/pixel"))
		  scale = .01;
	      else if (s.equals("10 cm/pixel"))
		  scale = .1;
	      else if (s.equals("1 m/pixel"))
		  scale = 1.0;
	      else if (s.equals("10 m/pixel"))
		  scale = 10.0;
	      topSlider.setScale(scale);
	      if (botSlider != null) botSlider.setScale(scale);
	      return(true);
	  }


      }

      else if (theEvent.target instanceof MenuItem) {

	  // Start animation

	  if (theEvent.target.equals (go)) {
	      go();
	      return(true);
	  }

	  // Stop animation

	  else if (theEvent.target.equals (stop)) {
	      stop();
	      return(true);
	  }

	  // Display only one slider
	  
	  else if (theEvent.target.equals(one)) {
	      sliderPanel.remove(botPanel);
	      botSlider = null;
	      botPanel = null;
	      sliderPanel.setLayout(new GridLayout(1,1));
	      resize(size().width, size().height - topPanel.size().height);
	      validate();
	      one.enable(false);
	      two.enable(true);
	      return(true);
	  }
	    
	  // Display two sliders

	  else if (theEvent.target.equals(two)) {
	      botSlider = new Slider(topSlider);
	      botPanel = makeSliderPanel(botSlider, topSlider.friction());
	      sliderPanel.add(botPanel);
	      sliderPanel.setLayout(new GridLayout(2,1));
	      resize(size().width, size().height + topPanel.size().height);
	      validate();
	      one.enable(true);
	      two.enable(false);
	      return(true);
	  }

      }
      
      // Elapsed time has been updated

      else if (theEvent.target.equals( timeField ) ) { 
	  topSlider.singleStep(timeField.getDoubleText());
	  if (botSlider != null)
	      botSlider.singleStep(timeField.getDoubleText());
	  return(true);
      }


      return(false);

    }


    //--------------------------------------------------
    // Starts the animation (if one isn't already happening)
    //--------------------------------------------------

    public synchronized void go () {
	if (animatorThread == null || !animatorThread.isAlive()) {
	    animatorThread = new Thread(this);
	    stop.enable(true);
	    go.enable(false);
	    animatorThread.start();
	}
    }
  

    //--------------------------------------------------
    // Stops the animation (if one is happening)
    //--------------------------------------------------

    public synchronized void stop () {
	if (animatorThread != null && animatorThread.isAlive()) {
	    stop.enable(false);
	    go.enable(true);
	    animatorThread.stop();
	}
    }



    //--------------------------------------------------
    // Stop any ongoing animation when we're disposed
    //--------------------------------------------------

    public void dispose () {
	stop();
	super.dispose();
    }

    
    //--------------------------------------------------
    // Controls animation of the sliding block
    //--------------------------------------------------

    public void run () {

	// The last time an update was done

	long lastTime = System.currentTimeMillis();

	while (true) {
	    
	    double time;
	    long currentTime;
	    boolean topGoing = true,
	            bottomGoing = true;

	    double limit = topSlider.limit();
	    if (botSlider != null) limit = Math.min(limit, botSlider.limit());

	    synchronized (this) {

	        // Figure out what the elapsed simulated time is

		currentTime = System.currentTimeMillis();
		time = timeField.getDoubleText();
		time += (currentTime - lastTime) / 1000.0 * speedratio;
		if (time > limit) time = limit;
		
		// Do the animation, but if nothing moves don't update
		// the time or the distance.

		topGoing = topSlider.animationStep(time);
		if (botSlider != null) 
		  bottomGoing = botSlider.animationStep(time);
		
		if (topGoing || (botSlider != null && bottomGoing)) {

		    timeField.setDoubleText(Math.round(time*100)/100.0);
		    lastTime = currentTime;

		}

	    }
	    
	    getToolkit().sync();
		
	    if (time >= limit) break;

	    Thread.yield();

	}
	
	stop();
	
    }


}

